using System;
using MSAvalon.Windows;
using MSAvalon.Windows.Media;
using MSAvalon.Windows.Input;
using MSAvalon.Windows.Controls;
using MSAvalon.Windows.Shapes;

namespace RubberBand
{
    /// <summary>
    /// Main window codebehind.
    /// </summary>

    public partial class Window1 : Window
    {
		// State for keeping track of the drag operation.

		Point startPoint;
		bool dragInProgress;


		// MouseLeftButtonDown handler
		//
		// Stores the initial location, captures the mouse and flags
		// the fact that a drag is in progress.
		// Ensures that the drag rectangle is visible and in the right
		// place.

		private void OnLBDown(object sender, MouseButtonEventArgs e)
		{
			dragInProgress = true;
			startPoint = e.GetPosition(mainPanel);
			CaptureMouse();

			dragRect.Visibility = Visibility.Visible;
			SetDragRectangleOpacity();
			SetDragRectangle(e);
		}


		// MouseLeftButtonUp handler
		//
		// Releases the mouse and stops the drag operation.
		// Hides the drag rectangle unless the 'Leave Visible'
		// checkbox is checked.

		private void OnLBUp(object sender, MouseButtonEventArgs e)
		{
			if (dragInProgress)
			{
				dragInProgress = false;
				if (!leaveVisible.Checked)
				{
					dragRect.Visibility = Visibility.Hidden;
				}
				ReleaseMouseCapture();
			}
		}


		// MouseMove handler
		//
		// If no drag is in progress this does nothing.  If
		// a drag is in progress, this updates the rectangle
		// position.

		private void OnMouseMove(object sender, MouseEventArgs e)
		{
			if (dragInProgress)
			{
				SetDragRectangle(e);
			}
		}


		// Sets the rectangle size and position appropriately.

		private void SetDragRectangle(MouseEventArgs e)
		{
			// We are not allowed to specify a negative width or height for a
			// Rectangle. We must therefore jig the X and Y coordinates of the
			// start and end locations of the drag to make sure everything
			// comes out positive.

			Point p = e.GetPosition(mainPanel);
			double w = p.X - startPoint.X;
			double h = p.Y - startPoint.Y;
			double x = startPoint.X;
			double y = startPoint.Y;
			if (w < 0)
			{
				x = p.X;
				w = -w;
			}
			if (h < 0)
			{
				y = p.Y;
				h = -h;
			}

			// Now we have the appropriate location and positive widths, set the size
			// and position.

			Canvas.SetLeft(dragRect, new Length(x));
			Canvas.SetTop(dragRect, new Length(y));
			dragRect.Width = new Length(w);
			dragRect.Height = new Length(h);
		}


		// ValueChanged handler for Opacity Slider
		//
		// Sets the opacity to the chosen value.

		private void OnOpacityChanged(object sender, ValueChangedEventArgs e)
		{
			SetDragRectangleOpacity();
		}


		// Makes the opacity of the drag rectangle's brush match the opacity
		// specified in the slide.

		private void SetDragRectangleOpacity()
		{
			// Due to a quirk in the way the UI gets initialized, the slider
			// raises its ValueChanged event before the drag rectangle gets
			// created, so we need to make sure we ignore that one!
			if (dragRect != null)
			{
				// Sadly we can't just set the existing brush's Opacity property. By
				// this time its IsChangeable property will be false.  So we have
				// to build a copy (which will be changeable), set the Opacity on
				// that, and then make that the rectangle's Fill brush.

				Brush b = dragRect.Fill.Copy();
				b.Opacity = opacitySlider.Value;
				dragRect.Fill = b;
			}
		}
	}
}